############################################################################################################
# VaRank 1.2.5                                                                                             #
#                                                                                                          #
# VaRank: a simple and powerful tool for ranking genetic variants                                          #
#                                                                                                          #
# Copyright (C) 2015 Veronique Geoffroy (veronique.geoffroy@inserm.fr)                                     #
#                    Jean Muller (jeanmuller@unistra.fr)                                                   #
#                                                                                                          #
# Please cite the following article:                                                                       #
# Geoffroy V.*, Pizot C.*, Redin C., Piton A., Vasli N., Stoetzel C., Blavier A., Laporte J. and Muller J. #
# VaRank: a simple and powerful tool for ranking genetic variants.                                         #
# PeerJ. 2015 (10.7717/peerj.796)                                                                          #
#                                                                                                          #
# This is part of VaRank source code.                                                                      #
#                                                                                                          #
# This program is free software; you can redistribute it and/or                                            #
# modify it under the terms of the GNU General Public License                                              #
# as published by the Free Software Foundation; either version 3                                           #
# of the License, or (at your option) any later version.                                                   #
#                                                                                                          #
# This program is distributed in the hope that it will be useful,                                          #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                                           #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                            #
# GNU General Public License for more details.                                                             #
#                                                                                                          #
# You should have received a copy of the GNU General Public License                                        #
# along with this program; If not, see <http://www.gnu.org/licenses/>.                                     #
############################################################################################################
 
##
## Filtering of all the existing ranking files (not already filtered).
##
## OUTPUTS: 	g_VaRank(vcfDir)/"family"_"patient"_filteredVariants.rankingByVar.tsv (1 par patient)
##

proc executeFilters {} {

    global g_VaRank
    global g_vcfINFOS

    puts "...writing all filtered ranking files ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"

    ## Remove all old filtered files
    ################################
    foreach filteredFile [glob -nocomplain $g_VaRank(vcfDir)/*filteredVariants*.tsv] {file delete -force $filteredFile}
    
    ## Filter every line with the user's options.
    ## Create the filtered files.
    #############################################################
    foreach rankFile [glob -nocomplain $g_VaRank(vcfDir)/*Variants.rankingBy*.tsv] {
	regsub "all(Variants)" $rankFile "filtered\\1" outputfile
	set OutputText ""
	foreach L [LinesFromFile $rankFile] {
	    if {[regexp "^## Barcode:" $L] || $L == ""} {append OutputText     $L; continue}
	    if {[regexp "^##"          $L] || $L == ""} {append OutputText "\n$L"; continue}

	    if {[regexp "^VariantID" $L]} {
		append OutputText "\n$L"
		set L [split $L "\t"]

		set i_mut       [lsearch -regexp $L "^Mut"];if {$i_mut == -1} {puts "Mut: column not found - Exit";exit}

		set i_clinic    [lsearch -regexp $L "^rsClinicalSignificance"];if {$i_clinic == -1} {puts "rsClinicalSignificance: column not found - Exit";exit}
		set i_valid     [lsearch -regexp $L "^rsValidation"];          if {$i_valid  == -1} {puts "rsValidation: column not found - Exit";exit}

		set i_cov       [lsearch -regexp $L "^TotalReadDepth"];   if {$i_cov       == -1} {puts "TotalReadDepth: column not found - Exit";exit}
		set i_read      [lsearch -regexp $L "^VarReadDepth"];     if {$i_read      == -1} {puts "ReadWithVar: column not found - Exit";exit}
		set i_percent   [lsearch -regexp $L "^%Reads_variation"]; if {$i_percent   == -1} {puts "%Reads_variation: column not found - Exit";exit}
		
		set i_rsMAF     [lsearch -regexp $L "^rsMAF"];            if {$i_rsMAF     == -1} {puts "rsMAF: column not found - Exit";exit}
		set i_rsMAFAllele [lsearch -regexp $L "^rsMAFAllele"];    if {$i_rsMAFAllele == -1} {puts "rsMAFAllele: column not found - Exit";exit}
		set i_espEAMAF  [lsearch -regexp $L "^espEAMAF"];         if {$i_espEAMAF  == -1} {puts "espEAMAF: column not found - Exit";exit}
		set i_espAAMAF  [lsearch -regexp $L "^espAAMAF"];         if {$i_espAAMAF  == -1} {puts "espAAMAF: column not found - Exit";exit}
		set i_espAllMAF [lsearch -regexp $L "^espAllMAF"];        if {$i_espAllMAF == -1} {puts "espAllMAF: column not found - Exit";exit}

		set i_1000g_AF     [lsearch -regexp $L "^1000g_AF"];              if {$i_1000g_AF     == -1} {puts "1000g_AF: column not found - Exit";exit}
		set i_1000g_AFR_AF [lsearch -regexp $L "^1000g_AFR_AF"];          if {$i_1000g_AFR_AF == -1} {puts "1000g_AFR_AF: column not found - Exit";exit}
		set i_1000g_SAS_AF [lsearch -regexp $L "^1000g_SAS_AF"];          if {$i_1000g_SAS_AF == -1} {puts "1000g_SAS_AF: column not found - Exit";exit}
		set i_1000g_EAS_AF [lsearch -regexp $L "^1000g_EAS_AF"];          if {$i_1000g_EAS_AF == -1} {puts "1000g_EAS_AF: column not found - Exit";exit}
		set i_1000g_EUR_AF [lsearch -regexp $L "^1000g_EUR_AF"];          if {$i_1000g_EUR_AF == -1} {puts "1000g_EUR_AF: column not found - Exit";exit}
		set i_1000g_AMR_AF [lsearch -regexp $L "^1000g_AMR_AF"];          if {$i_1000g_AMR_AF == -1} {puts "1000g_AMR_AF: column not found - Exit";exit}

		set i_exacAFRFreq [lsearch -regexp $L "^exacAFRFreq"];          if {$i_exacAFRFreq == -1} {puts "exacAFRFreq: column not found - Exit";exit}
		set i_exacSASFreq [lsearch -regexp $L "^exacSASFreq"];          if {$i_exacSASFreq == -1} {puts "exacSASFreq: column not found - Exit";exit}
		set i_exacEASFreq [lsearch -regexp $L "^exacEASFreq"];          if {$i_exacEASFreq == -1} {puts "exacEASFreq: column not found - Exit";exit}
		set i_exacAMRFreq [lsearch -regexp $L "^exacAMRFreq"];          if {$i_exacAMRFreq == -1} {puts "exacAMRFreq: column not found - Exit";exit}
		set i_exacNFEFreq [lsearch -regexp $L "^exacNFEFreq"];          if {$i_exacNFEFreq == -1} {puts "exacNFEFreq: column not found - Exit";exit}
		set i_exacFINFreq [lsearch -regexp $L "^exacFINFreq"];          if {$i_exacFINFreq == -1} {puts "exacFINFreq: column not found - Exit";exit}
		set i_exacOTHFreq [lsearch -regexp $L "^exacOTHFreq"];          if {$i_exacOTHFreq == -1} {puts "exacOTHFreq: column not found - Exit";exit}
		set i_exacAllFreq [lsearch -regexp $L "^exacAllFreq"];          if {$i_exacAllFreq == -1} {puts "exacAllFreq: column not found - Exit";exit}
		set i_exacFilter  [lsearch -regexp $L "^exacFilter"];           if {$i_exacFilter  == -1} {puts "exacFilter: column not found - Exit";exit}

		continue        
	    }
	    set L [split $L "\t"]
	    set cov [lindex $L $i_cov] 
	    set read [lindex $L $i_read]  
	    set percent [lindex $L $i_percent]
	    
	    #Filtering on sequencing data
	    #
	    if {$cov != "" && $cov != "NA" && $cov != "." && $cov != "0"} { 
		if {$cov < $g_VaRank(depthFilter)} {continue}

		if {$read != "" && $read != "NA" && $read != "."} {
		    if {$read    < $g_VaRank(readFilter)}        {continue}
		    if {$percent < $g_VaRank(readPercentFilter)} {continue}
		}
	    }

	    #Filtering on frequency data
	    #
	    if {$g_VaRank(freqFilter) != ""} {

		set TooFrequent 0

		#Global minor allele frequency (MAF)
		#
		#README_vcf_info_annotation.20141104
		#
		#The MAF is actually the second most frequent allele value. In other words, if there are 3 alleles, with frequencies of 0.50, 0.49, and 0.01, the MAF will be reported as 0.49. 
		#The current default global population is 1000Genome phase 1 genotype data from 1094 worldwide individuals, released in the May 2011 dataset.
		#http://www.ncbi.nlm.nih.gov/projects/SNP/docs/rs_attributes.html		

		set rsMAF [lindex $L $i_rsMAF]      
		set rsMAFAllele [lindex $L $i_rsMAFAllele] 
		set alt [lindex $L $i_mut]      

		set espEAMAF [lindex $L $i_espEAMAF]   
		set espAAMAF [lindex $L $i_espAAMAF]  
		set espAllMAF [lindex $L $i_espAllMAF]   

		#1000 genomes phase 3: Allele frequency by continental super population
		#The 2504 samples in the phase 3 release are from 26 populations which can be categorised into five super-populations by continent (listed below).  
		#
		#East Asian	EAS
		#South Asian	SAS
		#African	AFR
		#European	EUR
		#American	AMR
		#
		#These allele frequences were calculated by counting the AC and AN for all the individuals from a particular super population and using that
		#to calculate the AF. The info tag which represents the AFs are EAS_AF, EUR_AF, AFR_AF, AMR_AF and SAS_AF
		#
		#We have removed the genotypes for 31 individuals who have blood relationship with the 2504 samples in the main release. 
		#This was done to ensure we do not over estimate allele frequency. The 31 related samples are listed in 20140625_related_individuals.txt.
		#This has resulted in a small number of AC=0 sites from rare alleles only present in one or more of these 31 individuals.
		#
		#AF for multi-allelic variants are reported for each allele independently, separated by ",".
		
		set 1000g_AF [lindex $L $i_1000g_AF]  
		set 1000g_AFR_AF [lindex $L $i_1000g_AFR_AF]
		set 1000g_SAS_AF [lindex $L $i_1000g_SAS_AF]
		set 1000g_EAS_AF [lindex $L $i_1000g_EAS_AF] 
		set 1000g_EUR_AF [lindex $L $i_1000g_EUR_AF]
		set 1000g_AMR_AF [lindex $L $i_1000g_AMR_AF] 

		set exacAFRFreq [lindex $L $i_exacAFRFreq]
		set exacSASFreq [lindex $L $i_exacSASFreq]
		set exacEASFreq [lindex $L $i_exacEASFreq]
		set exacAMRFreq [lindex $L $i_exacAMRFreq]
		set exacNFEFreq [lindex $L $i_exacNFEFreq]
		set exacFINFreq [lindex $L $i_exacFINFreq]
		set exacOTHFreq [lindex $L $i_exacOTHFreq]
		set exacAllFreq [lindex $L $i_exacAllFreq]
		set exacFilter  [lindex $L $i_exacFilter]

		#set L_Fields2Filter [list $rsMAF $espAllMAF $espEAMAF $espAAMAF]
		if {$rsMAFAllele == $alt} {
		    set L_Fields2Filter [list $rsMAF $espAllMAF $1000g_AF $espEAMAF $espAAMAF $1000g_AFR_AF $1000g_SAS_AF $1000g_EAS_AF $1000g_EUR_AF $1000g_AMR_AF]
		} else {
		    set L_Fields2Filter [list $espAllMAF $1000g_AF $espEAMAF $espAAMAF $1000g_AFR_AF $1000g_SAS_AF $1000g_EAS_AF $1000g_EUR_AF $1000g_AMR_AF]
		}
		if {$exacFilter == "PASS"} {lappend L_Fields2Filter {*}{$exacAFRFreq $exacSASFreq $exacEASFreq $exacAMRFreq $exacNFEFreq $exacFINFreq $exacOTHFreq $exacAllFreq}}

		foreach freq $L_Fields2Filter {
		    if {[set g_VaRank(metrics)]=="fr"} {
			#puts "Filters replacing $freq"
			regsub {\,} $freq "." freq
			#puts "Filters: $freq"
		    }

		    if {$freq != "NA" && $freq != "" && $freq>=$g_VaRank(freqFilter)} {set TooFrequent 1;break}
		}

		if {$TooFrequent} {continue}
	    }

	    #Filtering on sequencing data
	    #
	    #Here we need to improve filter and add also frequency
	    if {$g_VaRank(rsFilter) == "removeNonPathoRS"} {
		set valid [lindex $L $i_valid]  
		set clinic [lindex $L $i_clinic] 

		# Vero, 2014/11/07  ----  "Clinical significance"
		# Alamut developpers communication:   
		# "Depuis le 9 septembre 2014, notre source de données dbSNP n’est plus le NCBI mais Ensembl.
		# En raison de problèmes de qualité rencontrés dans les fichiers XML fournis par le NCBI, nous avons décidé d’utiliser l’API d’Ensembl comme source de données dbSNP."
		# La signification clinique fournie par Ensembl correspond à la signification clinique donnée par la base de données ClinVar (NCBI).
		#
		# http://www.ensembl.org/info/genome/variation/data_description.html#clin_significance
		# http://www.ncbi.nlm.nih.gov/clinvar/docs/clinsig/
		#
		# [association] 	         association	
		# [benign] 	                 benign	        
		# [confers sensitivity] 	 confers sensitivity
		# [drug response] 	 	 drug response
		# [likely benign] 	 	 likely benign	
		# [likely pathogenic]  		 likely pathogenic
		# [not provided] 	         not provided	
		# [other] 	       		 other
		# [pathogenic] 	                 pathogenic
		# [protective] 	        	 protective
		# [risk factor] 	         risk factor
		# [uncertain significance] uncertain significance
 
		# Excluding validated benign variants (Keeping the potential pathogenic ones)
		if {[regexp -nocase "benign|^NA$" $clinic]} {
		    #Keeping only the ones with 2 validation or more
		    if {[regsub -all "/|;" $valid "" toto] >= 1 || [regexp -nocase "^yes$" $valid]} {continue}
		}
	    }

	    append OutputText "\n[join $L "\t"]"
	}
	if {[regexp "rankingByVar" $outputfile]} {
	    ReplaceTextInFile $OutputText $outputfile
	    continue
	}

	## Ranking by gene have to be done once again after filtering
	regexp "fam\[0-9\]+_(.+)_allVariants.rankingBy" $rankFile match patient
	set OutputText2 ""
	set lGenes ""
	set lVariantsRankingByGene ""
	set lID ""
	foreach L [split $OutputText "\n"] {
	    if {$L == ""} {continue}
	    if {[regexp "^## Barcode:" $L]} {append OutputText2     $L; continue}
	    if {[regexp "^##"          $L]} {append OutputText2 "\n$L"; continue}
	    if {[regexp "^VariantID" $L]} {
		append OutputText2 "\n$L"
		set Ls [split $L "\t"]
		set i_gene  [lsearch -regexp $Ls "^Gene$"];            if {$i_gene  == -1} {puts "Gene: column number not found - Exit"; exit}
		set i_score [lsearch -regexp $Ls "^VaRank_VarScore$"]; if {$i_score == -1} {puts "VaRank_VarScore: column number not found - Exit"; exit}
		continue
	    }
	    set Ls [split $L "\t"]
	    set id [lindex $Ls 0]     
	    set gene [lindex $Ls $i_gene] 
	    if {$gene == "NA"} {continue}

	    if {[lsearch -exact $lGenes "$gene"] == -1} {
		lappend lGenes $gene
	    }
	    set l($id) $L
		
	    set score [lindex $Ls $i_score] 
	    
	    lappend lVariants($gene) "$id $score"
	}
	
	foreach gene $lGenes {
	    set liste {}
	    foreach duoIDScore $lVariants($gene) {
		set id [lindex $duoIDScore 0]
		lappend liste $id
	    }
	    set maxScore1 [lindex [lindex $lVariants($gene) 0] 1]
	    if {[llength $lVariants($gene)] == 1} {
		set BestDuoScore "[expr {$maxScore1*2}]"
	    } else {
		set bestID [lindex [lindex $lVariants($gene) 0] 0]
		if {[regexp "$patient:(\[^: \]+):" $g_vcfINFOS($bestID) match homhtz] && [regexp -nocase "hom" $homhtz]} {
		    set BestDuoScore "[expr {$maxScore1*2}]"
		} else {
		    set maxScore2 [lindex [lindex $lVariants($gene) 1] 1]
		    set BestDuoScore "[expr {$maxScore1+$maxScore2}]"
		}
	    }
	    lappend lVariantsRankingByGene "{$liste} $BestDuoScore"
	}
	if {[info exists lVariantsRankingByGene]} {
	    set lVariantsRankingByGene [lsort -command DescendingSortOnElement1 $lVariantsRankingByGene]
	}    
	foreach val $lVariantsRankingByGene {
	    set lID [lindex $val 0]
	    foreach id $lID {
		append OutputText2 "\n$l($id)"
	    }
	}

	ReplaceTextInFile "$OutputText2" $outputfile
	
	## Very important 
	if {[info exists l]} {unset l}
	if {[info exists lVariants]} {unset lVariants}
    }
    return
}


